<?php

namespace Drupal\map_object_field\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\map_object_field\MapObject\MapObjectService;
use Drupal\map_object_field\Service\MapObjectLibInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;

/**
 * Plugin implementation of the 'map_object_field_default' formatter.
 *
 * @FieldFormatter(
 *   id = "map_object_field_default",
 *   label = @Translation("Map Object Field default"),
 *   field_types = {
 *     "map_object_field"
 *   }
 * )
 */
class MapObjectFieldDefaultFormatter extends FormatterBase {

  /**
   * The map object service.
   *
   * @var \Drupal\map_object_field\MapObject\MapObjectService
   */
  protected $mapObjectService;

  /**
   * The map object lib service.
   *
   * @var \Drupal\map_object_field\Service\MapObjectLibInterface
   */
  protected $mapObjectLibService;

  /**
   * Constructs a MapObjectFieldDefaultFormatter object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\map_object_field\MapObject\MapObjectService $map_object_service
   *   The map object service.
   * @param \Drupal\map_object_field\Service\MapObjectLibInterface $map_object_lib
   *   The map object lib service.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    MapObjectService $map_object_service,
    MapObjectLibInterface $map_object_lib
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->mapObjectService = $map_object_service;
    $this->mapObjectLibService = $map_object_lib;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, string $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('map_object.service'),
      $container->get('map_object_field_lib')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($items as $delta => $item) {
      /** @var \Drupal\Core\Entity\EntityInterface $entity */
      $entity = $item->getEntity();
      $map_object_field_data = $this->mapObjectService->getMapObjectsByFieldDataAsString(
        $entity->getEntityType()->id(),
        $entity->id(),
        $entity->getEntityType()->isRevisionable() ? $entity->getRevisionId() : $entity->id(),
        $delta
      );
      $element = [
        '#theme' => 'map_default_format',
        '#name' => $item->map_object_name,
        '#overlays' => $map_object_field_data,
        '#mapOptions' => [
          'center-lat' => $item->map_center_lat,
          'center-lng' => $item->map_center_lng,
          'zoom' => $item->map_zoom,
          'map-type' => $item->map_type,
        ],
        '#mapWidth' => $this->getSetting('map_width'),
        '#mapHeight' => $this->getSetting('map_height'),
      ];

      foreach ($this->mapObjectLibService->getLibrariesForFormatter() as $lib) {
        $element['#attached']['library'][] = $lib;
      }
      $elements[$delta] = $element;
    }
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'map_width' => '100%',
      'map_height' => '400px',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    $settings = $this->getSettings();

    $elements['map_width'] = [
      '#title' => $this->t('Map width'),
      '#type' => 'textfield',
      '#default_value' => $settings['map_width'] ?? '100%',
      '#size' => 10,
    ];

    $elements['map_height'] = [
      '#title' => $this->t('Map height'),
      '#type' => 'textfield',
      '#default_value' => $settings['map_height'] ?? '400px',
      '#size' => 10,
    ];
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $settings = $this->getSettings();
    $summary[] = $this->t(
      'Map width: @map_width',
      ['@map_width' => $settings['map_width']]
    );
    $summary[] = $this->t(
      'Map height: @map_height',
      ['@map_height' => $settings['map_height']]
    );
    return $summary;
  }

}
